import 'dart:developer';

import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/helpers/database/database_helper.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:sqflite/sqflite.dart';

abstract class LicenseLocalDataSource {
  Future<void> init();
  Future<void> setByKey(final List<String> key, final String display);
  Future<bool> isNewVersion();
  Future<String?> g(String key);
  Future<void> s<T>(String key, T value);
  Future<void> d(String key);
  Future<void> c();
}

class LicenseLocalDataSourceImpl implements LicenseLocalDataSource {
  static final LicenseLocalDataSourceImpl _instance = LicenseLocalDataSourceImpl._internal();
  factory LicenseLocalDataSourceImpl() => _instance;
  LicenseLocalDataSourceImpl._internal();


  Future<void> init() async {
    await DatabaseStorage.instance.init();
  }

  Future<void> setByKey(final List<String?> key, final String display) async {
    if (key.isEmpty) return;
    for (var element in key) {
      if (element != null) {
        await s(element, {
          'display': display,
          'expirationDate': '',
          'startDate': '',
          'quantity': '',
        });
      }
    }
  }

  Future<bool> isNewVersion() async {
    var response = await DatabaseStorage.database.query(tableLicense, where: 'key = ?', whereArgs: [KeychainStorageKey.isNewVersion.value], columns: ['display']);
    if (response.isEmpty) {
      return false;
    }
    return response.first['display'] == 'true';
  }

  Future<String?> g(String key) async {
    var response = await DatabaseStorage.database.query(tableLicense, where: 'key = ?', whereArgs: [key]);
    if (response.isEmpty) {
      return null;
    }
    return response.first.toString();
  }

  Future<void> s<T>(String key, T value) async {
    value as Map<String, dynamic>;

    await DatabaseStorage.database.insert(
        tableLicense,
        {
          'key': key,
          'display': value['display'],
          'expirationDate': value['expirationDate'],
          'startDate': value['startDate'],
          'quantity': value['quantity'],
        },
        conflictAlgorithm: ConflictAlgorithm.replace);
    return;
  }

  Future<void> d(String key) async {
    await DatabaseStorage.database.delete(tableLicense, where: 'key = ?', whereArgs: [key]);
  }

  Future<void> c() async {
    await DatabaseStorage.database.delete(tableLicense);
  }
}
